@@ -2,7 +2,7 @@
 #
 # (c) 2002 - 2016, Arthur Corliss <corliss@digitalmages.com>,
 #
-# $Id: lib/Parse/PlainConfig.pm, 3.01 2016/05/13 20:34:25 acorliss Exp $
+# $Id: lib/Parse/PlainConfig.pm, 3.02 2016/05/26 13:04:45 acorliss Exp $
 #
 #    This software is licensed under the same terms as Perl, itself.
 #    Please see http://dev.perl.org/licenses/ for more information.
@@ -23,14 +23,13 @@ use strict;
 use warnings;
 use vars qw($VERSION);
 
-($VERSION) = ( q$Revision: 3.01 $ =~ /(\d+(?:\.(\d+))+)/sm );
+($VERSION) = ( q$Revision: 3.02 $ =~ /(\d+(?:\.(\d+))+)/sm );
 
 use Class::EHierarchy qw(:all);
 use Parse::PlainConfig::Constants qw(:all);
 use Parse::PlainConfig::Settings;
 use Text::ParseWords;
 use Text::Tabs;
-use Carp;
 use Fcntl qw(:seek :DEFAULT);
 use Paranoid;
 use Paranoid::Debug;
@@ -120,9 +119,11 @@ sub _initialize {
                 }
 
                 # Store property regex
-                $settings->store( 'property regexes',
+                $settings->store(
+                    'property regexes',
                     $_,
-                    qr#(\s*)(\Q$_\E)\s*\Q@{[ $settings->delimiter ]}\E\s*(.*)#s );
+                    qr#(\s*)(\Q$_\E)\s*\Q@{[ $settings->delimiter ]}\E\s*(.*)#s
+                    );
             }
         }
     }
@@ -233,7 +234,8 @@ sub get {
     if ( defined $p ) {
         $valp = scalar grep /^\Q$p\E$/s, $obj->propertyNames;
     }
-    carp pdebug( 'specified invalid parameter name: %s', PPCDLEVEL1, $p )
+    $obj->error(
+        pdebug( 'specified invalid parameter name: %s', PPCDLEVEL1, $p ) )
         unless $valp;
 
     pOut();
@@ -260,7 +262,8 @@ sub set {
     if ( defined $p ) {
         $valp = scalar grep /^\Q$p\E$/s, $obj->propertyNames;
     }
-    carp pdebug( 'specified invalid parameter name: %s', PPCDLEVEL1, $p )
+    $obj->error(
+        pdebug( 'specified invalid parameter name: %s', PPCDLEVEL1, $p ) )
         unless $valp;
 
     if ($valp) {
@@ -297,12 +300,13 @@ sub _snarfBlock (\@\$\$$) {
     my $pref       = shift;
     my $vref       = shift;
     my $settings   = shift;
+    my $obj        = $settings->parent;
     my %regex      = $settings->propertyRegexes;
     my %pregex     = $settings->prototypeRegexes;
     my %propTypes  = $settings->propertyTypes;
     my %prototypes = $settings->prototypes;
     my $subi       = $settings->subindentation;
-    my ( $rv, $indent, $prop, $proto, $trailer, $iwidth, $line );
+    my ( $rv, $indent, $prop, $proto, $trailer, $iwidth, $line, $preg );
 
     pdebug( 'entering', PPCDLEVEL2 );
     pIn();
@@ -334,33 +338,40 @@ sub _snarfBlock (\@\$\$$) {
         if ( defined $prop and length $prop ) {
 
             if ( exists $regex{$prop} ) {
-                pdebug(
-                    'token (%s) for prototype (%s) attempted to override property',
-                    PPCDLEVEL1, $prop, $proto
-                    );
+                $obj->error(
+                    pdebug(
+                        'token (%s) for prototype (%s) attempted to override property',
+                        PPCDLEVEL1,
+                        $prop,
+                        $proto
+                        ) );
                 $rv = 0;
             } else {
 
-                # Make sure they haven't been previously defined,
-                # or if they have, they match the same type
                 if ( exists $propTypes{$prop} ) {
+
+                    # Make sure they haven't been previously defined,
+                    # or if they have, they match the same type
                     unless ( $propTypes{$prop} == $prototypes{$proto} ) {
                         $rv = 0;
-                        pdebug(
-                            'prototype mismatch with previous declaration: %s',
-                            PPCDLEVEL1, $proto
-                            );
+                        $obj->error(
+                            pdebug(
+                                'prototype mismatch with previous declaration: %s',
+                                PPCDLEVEL1,
+                                $proto
+                                ) );
                         pdebug( 'current type: %s prototype: %s',
                             PPCDLEVEL1, $propTypes{$prop},
                             $prototypes{$proto} );
                     }
                 } else {
+
+                    # Create a new property
                     pdebug( 'creating property based on prototype %s: %s',
                         PPCDLEVEL3, $proto, $prop );
 
-                    # This will cause some croaking... need to fix
                     $rv = _declProp(
-                        $settings->parent,
+                        $obj,
                         CEH_PUB | (
                             $prototypes{$proto} == PPC_HDOC
                             ? PPC_SCALAR
@@ -370,15 +381,31 @@ sub _snarfBlock (\@\$\$$) {
                         );
 
                     # Record the prop type
-                    $settings->store( 'property types',
-                        $prop, $propTypes{$prop} = $prototypes{$proto} )
-                        if $rv;
+                    if ($rv) {
+                        $settings->store( 'property types',
+                            $prop, $propTypes{$prop} = $prototypes{$proto} );
+                        $preg =
+                            $settings->retrieve( 'prototype registry',
+                            $proto );
+                        $preg = [] unless defined $preg;
+                        push @$preg, $prop;
+                        $settings->store( 'prototype registry',
+                            $proto => $preg );
+                    } else {
+                        $obj->error(
+                            pdebug(
+                                'failed to declare prototype: %s %s',
+                                PPCDLEVEL1, $proto, $prop
+                                ) );
+                    }
                 }
             }
-
         } else {
-            pdebug( 'invalid token used for prototype %s: %s',
-                PPCDLEVEL1, $proto, $prop );
+            $obj->error(
+                pdebug(
+                    'invalid token used for prototype %s: %s', PPCDLEVEL1,
+                    $proto,                                    $prop
+                    ) );
             $rv = 0;
         }
     }
@@ -402,7 +429,8 @@ sub _snarfBlock (\@\$\$$) {
             }
 
             # Error out if we never found the marker
-            carp pdebug( 'failed to find the here doc marker', PPCDLEVEL1 )
+            $obj->error(
+                pdebug( 'failed to find the here doc marker', PPCDLEVEL1 ) )
                 unless $rv;
 
         } else {
@@ -536,7 +564,7 @@ sub parse {
 
         # Handle "include" statements
         if ( $lines[0] =~ /^\s*include\s+(.+?)\s*$/s ) {
-            $glob = new Paranoid::Glob globs => [ $1 ];
+            $glob = new Paranoid::Glob globs => [$1];
             shift @lines;
             $rv = 0 unless $obj->read($glob);
             next;
@@ -588,8 +616,10 @@ sub read {
             $rv = $obj->parse(@lines);
             pdebug( 'errors parsing %s', PPCDLEVEL1, $source ) unless $rv;
         } else {
-            pdebug( 'failed to read %s: %s',
-                PPCDLEVEL1, $source, Paranoid::ERROR() );
+            $obj->error(
+                pdebug(
+                    'failed to read %s: %s', PPCDLEVEL1,
+                    $source,                 Paranoid::ERROR() ) );
         }
 
     } elsif ( ref $source eq 'Paranoid::Glob' ) {
@@ -605,8 +635,10 @@ sub read {
             $rv = $obj->parse(@lines);
             pdebug( 'errors parsing %s', PPCDLEVEL1, $source ) unless $rv;
         } else {
-            pdebug( 'failed to read %s: %s',
-                PPCDLEVEL1, $source, Paranoid::ERROR() );
+            $obj->error(
+                pdebug(
+                    'failed to read %s: %s', PPCDLEVEL1,
+                    $source,                 Paranoid::ERROR() ) );
         }
     }
 
@@ -617,6 +649,11 @@ sub read {
 }
 
 sub reset {
+
+    # Purpose:  Resets configuration state to defaults
+    # Returns:  Boolean
+    # Usage:    $rv = $obj->reset;
+
     my $obj       = shift;
     my $settings  = $obj->settings;
     my %propTypes = $settings->propertyTypes;
@@ -642,6 +679,59 @@ sub reset {
     return $rv;
 }
 
+sub prototyped {
+
+    # Purpose:  Returns a list of properties that were created with
+    #           prototypes
+    # Returns:  Array
+    # Usage:    @protos = $obj->prototyped;
+
+    my $obj   = shift;
+    my $proto = shift;
+    my ( %preg, @prval );
+
+    pdebug( 'entering w/%s', PPCDLEVEL1, $proto );
+    pIn();
+
+    %preg = $obj->settings->property('prototype registry');
+
+    if ( defined $proto and length $proto ) {
+        if ( exists $preg{$proto} ) {
+            @prval = @{ $preg{$proto} };
+        } else {
+            pdebug( 'no prototype properties declared w/%s',
+                PPCDLEVEL2, $proto );
+        }
+    } else {
+        pdebug( 'dumping all declared prototyped properties', PPCDLEVEL2 );
+        foreach ( keys %preg ) { push @prval, @{ $preg{$_} } }
+    }
+
+    pOut();
+    pdebug( 'leaving w/%s', PPCDLEVEL1, @prval );
+
+    return @prval;
+}
+
+sub error {
+
+    # Purpose:  Sets/gets the last error message
+    # Returns:  Scalar/undef
+    # Usage:    $errStr = $obj->error;
+    # Usage:    $errStr = $obj->error($msg);
+
+    my $obj = shift;
+    my $msg = shift;
+
+    if ( defined $msg ) {
+        $obj->settings->property( 'error', $msg );
+    } else {
+        $msg = $obj->settings->property('error');
+    }
+
+    return $msg;
+}
+
 1;
 
 __END__
@@ -652,7 +742,7 @@ Parse::PlainConfig - Configuration file class
 
 =head1 VERSION
 
-$Id: lib/Parse/PlainConfig.pm, 3.01 2016/05/13 20:34:25 acorliss Exp $
+$Id: lib/Parse/PlainConfig.pm, 3.02 2016/05/26 13:04:45 acorliss Exp $
 
 =head1 SYNOPSIS
 
@@ -1016,16 +1106,29 @@ following parameters depending on the size of your config files:
 This method purges the contents of all parameters and prototypes, then applies
 the default settings as found in B<__DATA__>.
 
+=head2 prototyped
+
+    @protos = $config->prototyped;
+    @protos = $config->prototyped($preamble);
+
+This method returns a list of properties that were defined as the result of
+prototypes.  With no arguments it returns all properties that were defined.
+With an argument it returns only those properties that were defined by that
+specific prototype preamble.
+
+=head2 error
+
+    $errStr = $config->error;
+
+Returns the last error that occured.  Note that this isn't reset between
+method invocations.
+
 =head1 DEPENDENCIES
 
 =over
 
 =item o
 
-L<Carp>
-
-=item o
-
 L<Class::EHierarchy>
 
 =item o
